/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * Portions of this software are based upon public domain software
 * originally written at the National Center for Supercomputing Applications,
 * University of Illinois, Urbana-Champaign.
 */

#include "ap_config.h"
#include "httpd.h"
#include "ap_iol.h"
#include "iol_socket.h"
#include <apr_network_io.h>
#include <apr_file_io.h>

#include <sys/types.h>
#include <sys/uio.h>

#define FD_NONBLOCKING_SET	(1)

typedef struct {
    ap_iol iol;
    ap_socket_t *sock;
} iol_socket;

static ap_status_t os2_setopt(ap_iol *viol, ap_iol_option opt,
                               const void *value)
{
    switch (opt) {
    case AP_IOL_TIMEOUT:
        ap_setsocketopt(((iol_socket *)viol)->sock, APR_SO_TIMEOUT, 
                        *(const int *)value * AP_USEC_PER_SEC);
	break;
    default:
	return APR_EINVAL;
    }
    return APR_SUCCESS;
}

static ap_status_t os2_getopt(ap_iol *viol, ap_iol_option opt, void *value)
{
    switch (opt) {
    case AP_IOL_TIMEOUT:
    /* There's no APR getopt yet but nothing uses this option anyway */
    default:
	return APR_EINVAL;
    }
    return APR_SUCCESS;
}

static ap_status_t os2_write(ap_iol *viol, const char *buf, ap_size_t size, ap_ssize_t *nbytes)
{
    *nbytes = size;
    return ap_send(((iol_socket *)viol)->sock, buf, nbytes);
}

static ap_status_t os2_writev(ap_iol *viol, const struct iovec *vec, ap_size_t nvec, ap_ssize_t *nbytes)
{
    return ap_sendv(((iol_socket *)viol)->sock, vec, nvec, nbytes);
}

static ap_status_t os2_read(ap_iol *viol, char *buf, ap_size_t size, ap_ssize_t *nbytes)
{
    *nbytes = size;
    return ap_recv(((iol_socket *)viol)->sock, buf, nbytes);
}

static ap_status_t os2_close(ap_iol *viol)
{
    iol_socket *iol = (iol_socket *)viol;
    int rv;

    rv = ap_close_socket(iol->sock);
    free(iol);
    return rv;
}

static ap_status_t os2_shutdown(ap_iol *viol, int how)
{
    iol_socket *iol = (iol_socket *)viol;

    return ap_shutdown(iol->sock, how);
}
    
static const ap_iol_methods socket_methods = {
    os2_close,
    os2_write,
    os2_writev,
    os2_read,
    os2_setopt,
    os2_getopt,
    NULL,
    os2_shutdown
};

ap_iol *os2_attach_socket(ap_socket_t *sock)
{
    iol_socket *iol;

    iol = malloc(sizeof(iol_socket));
    iol->iol.methods = &socket_methods;
    iol->sock = sock;
    return (ap_iol *)iol;
}
